
#include "EOSX.h"
#include "EOSXparamids.h"
#include "EOSXcontroller.h"

#include "pluginterfaces/base/ibstream.h"
#include "pluginterfaces/vst/ivstparameterchanges.h"
#include "pluginterfaces/vst/ivstevents.h"
#include "pluginterfaces/base/futils.h"

#include <stdio.h>
#include "QSocketUDP.h"

// **************************************************************************
namespace Steinberg {
namespace Vst {

// This plug-ins GUIDv6.5/ShTc-format for the framework and its factories.
FUID EOSX::cid(0x53686164, 0x6F775465, 0x63000000, 0x454F5358);


// **************************************************************************
// constructors and destructors
// **************************************************************************

// --------------------------------------------------------------------------
// generic constructor
EOSX::EOSX() : bBypass (false)
{
	// register its editor class
	setControllerClass (EOSXController::cid);
	mp_Socket = new ShadowTec::CQSocketUDP();
}

// --------------------------------------------------------------------------
// virtual destructor
EOSX::~EOSX()
{
	delete mp_Socket;
}


// **************************************************************************
// VST initialization
// **************************************************************************

// --------------------------------------------------------------------------
// get interface-pointer by iid
tresult PLUGIN_API EOSX::queryInterface(const TUID _iid, void** obj)
{
	QUERY_INTERFACE(_iid, obj, EOSX::cid, EOSX)
		return AudioEffect::queryInterface(_iid, obj);
}

// --------------------------------------------------------------------------
// called at first after constructor
tresult PLUGIN_API EOSX::initialize (FUnknown* context)
{
	// always initialize the parent
	tresult result = AudioEffect::initialize (context);
	// break on error
	if (result != kResultOk)
		return result;
	// create busses
	addAudioInput (STR16 ("Audio In"), SpeakerArr::kMono);
	addAudioOutput (STR16 ("Audio Out"), SpeakerArr::kMono);
	addEventInput(STR16("CueFire"), 1);
	// we're finished, tell caller
	return kResultOk;
}


// **************************************************************************
// Automatization and serialization
// **************************************************************************

// --------------------------------------------------------------------------
// load model from bitstream
tresult PLUGIN_API EOSX::setState(IBStream* state)
{
	importSaveState(state, saveState);
	bBypass = saveState.bypass > 0;
	return kResultOk;
}

// --------------------------------------------------------------------------
// save model into bitstream
tresult PLUGIN_API EOSX::getState(IBStream* state)
{
	saveState.bypass = bBypass ? 1 : 0;
	exportSaveState(state, saveState);
	return kResultOk;
}

// --------------------------------------------------------------------------
// automate parameter change
tresult PLUGIN_API EOSX::process(ProcessData& data)
{
	// Read input parameter changes
	IParameterChanges* paramChanges = data.inputParameterChanges;
	if (paramChanges)
	{
		int32 numParamsChanged = paramChanges->getParameterCount();
		// for each parameter we read and process change
		for (int32 i = 0; i < numParamsChanged; i++)
			ParamChangeRead(paramChanges->getParameterData(i));
	}

	// process audio
	if (data.numInputs == 0 || data.numOutputs == 0)
		return kResultOk; // nothing to do

	// NYI: we suppose in this example that we have the same input
	// channel count than the output
	int32 numChannels = data.inputs[0].numChannels;

	// get audio buffers
	float** in = data.inputs[0].channelBuffers32;
	float** out = data.outputs[0].channelBuffers32;

	if (data.inputs[0].silenceFlags != 0)
	{
		// mark output silence too
		data.outputs[0].silenceFlags = data.inputs[0].silenceFlags;

		int32 sampleFrames = data.numSamples;

		// the Plug-in has to be sure that if it sets the flags silence
		// that the output buffer are  clear
		for (int32 i = 0; i < numChannels; i++)
		{
			// do not need to be cleared if the buffers are the same
			// (in this case input buffer are already cleared by the host)
			if (in[i] != out[i])
				memset(out[i], 0, sampleFrames * sizeof(float));
		}

		// nothing to do at this point
		return kResultOk;
	}

	// mark our outputs has not silent
	data.outputs[0].silenceFlags = 0;

	// pure audio bypass, will remove audio later
	int32 sampleFrames = data.numSamples;
	for (int32 i = 0; i < numChannels; i++)
	{
		// do not need to be copied if the buffers are the same
		if (in[i] != out[i])
			memcpy(out[i], in[i], sampleFrames * sizeof(float));
	}

	return kResultOk;
}

// --------------------------------------------------------------------------
// apply parameter change
tresult EOSX::ParamChangeRead(IParamValueQueue* paramQueue)
{
	if (!paramQueue)
		return kResultOk;

	int32 offsetSamples;
	double value;
	int32 numPoints = paramQueue->getPointCount();
	if (!paramQueue->getPoint(numPoints - 1, offsetSamples, value) ==
		kResultTrue)
		return kResultOk;

	switch (paramQueue->getParameterId())
	{
	case kBypassId:
		bBypass = (value > 0.5f);
		break;
	case kCueListId:
		saveState.currentCue.list = FromNormalized<ParamValue>(value, 999);
		break;
	case kCueId:
		saveState.currentCue.cue = FromNormalized<ParamValue>(value, 999);
		break;
	case kSubCueId:
		saveState.currentCue.subcue = FromNormalized<ParamValue>(value, 999);
		break;
	}
	return kResultOk;
}


// **************************************************************************
// static class functions and fields
// **************************************************************************

// --------------------------------------------------------------------------
// export values into stream
tresult PLUGIN_API EOSX::exportSaveState(IBStream* state,
	EOSXsavestate const& rState)
{
	EOSXsavestate* pState = const_cast<EOSXsavestate*>(&rState);

#if BYTEORDER == kBigEndian
	EOSXsavestate tmp(rState);
	pState = &tmp;
	SWAP_32(tmp.bypass)
	SWAP_32(tmp.currentCue.list)
	SWAP_32(tmp.currentCue.cue)
	SWAP_32(tmp.currentCue.subcue)
#endif

	state->write(&pState->bypass,
		sizeof(rState.bypass));
	state->write(&pState->currentCue.list,
		sizeof(rState.currentCue.list));
	state->write(&pState->currentCue.cue,
		sizeof(rState.currentCue.cue));
	state->write(&pState->currentCue.subcue,
		sizeof(rState.currentCue.subcue));

	return kResultTrue;
}

// --------------------------------------------------------------------------
// import values from stream
tresult PLUGIN_API EOSX::importSaveState(IBStream* state,
	EOSXsavestate& rState)
{
	// called when we load a preset, the model has to be reloaded
	if ((state->read(&rState.bypass, sizeof(saveState.bypass)) != kResultOk)
		|| (state->read(&rState.currentCue.list,
			sizeof(rState.currentCue.list)) != kResultOk)
		|| (state->read(&rState.currentCue.cue,
			sizeof(rState.currentCue.cue)) != kResultOk)
		|| (state->read(&rState.currentCue.subcue,
			sizeof(rState.currentCue.subcue)) != kResultOk)
		)
	{
		return kResultFalse;
	}

#if BYTEORDER == kBigEndian
	SWAP_32(saveState.bypass)
		SWAP_32(saveState.currentCue.list)
		SWAP_32(saveState.currentCue.cue)
		SWAP_32(saveState.currentCue.subcue)
#endif
		return kResultTrue;
}


// **************************************************************************
}} // namespaces
